package org.wronka.matt;

import org.wronka.matt.Okc.OKConnection;
import org.wronka.matt.Okc.Mail;
import org.wronka.matt.Okc.Message;
import org.wronka.matt.util.Convert;
import java.util.Random;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.io.IOException;
import java.io.Writer;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.InputStreamReader;
import java.io.InputStream;
import java.io.BufferedReader;

public class FetchOK {
	OKConnection okc;

	// Options
	int remotemailbox	= 1 ;
	int requestedpage	= 0 ;
	String user	= "";
	String password	= "";
	String recipient	= "";
	String subject	= "";
	String domain	= "";
	String localmailbox	= "";
	boolean bNewOnly = false;
	boolean bFetch = false;
	boolean bVerbose = false;

	/**
	 * TODO:
	 * If mailbox ends in a '/', it is assumed to be Maildir.
	 * Otherwise, it is assumed to be mbox.
	 *
	 * Currently:  we use mbox
	 **/
	public static void main(String[] Args) {
		new FetchOK().run(Args);

	}

	protected void parseArgs(String[] Args) throws Exception {
		int i=0;
		for ( ; i < Args.length ; ++i ) {
			if(Args[i].charAt(0) != '-' || Args[i].length()<2) {
				throw new Exception("Bad Argument");
			}
			char c = Args[i].charAt(1);
			boolean bHasArgAppended = (Args[i].length()>2);
			boolean bHasMoreArgs = (Args.length>i+1);
			switch(c) {
			case 'g':
				if(bHasArgAppended) throw new Exception("Bad Argument");
				bFetch = true;
				break;
			case 'b':
				if(!(bHasArgAppended || bHasMoreArgs)) throw new Exception("Bad Argument");
				if(bHasArgAppended) {
					remotemailbox = Integer.parseInt(Args[i].substring(2));
				} else {
					remotemailbox = Integer.parseInt(Args[++i]);
				}
				break;
			case 'P':
				if(!(bHasArgAppended || bHasMoreArgs)) throw new Exception("Bad Argument");
				if(bHasArgAppended) {
					requestedpage = Integer.parseInt(Args[i].substring(2));
				} else {
					requestedpage = Integer.parseInt(Args[++i]);
				}
				break;
			case 'n':
				if(bHasArgAppended) throw new Exception("Bad Argument");
				bNewOnly = true;
				break;
			case 'f':
				if(!(bHasArgAppended || bHasMoreArgs)) throw new Exception("Bad Argument");
				if(bHasArgAppended) {
					localmailbox = Args[i].substring(2);
				} else {
					localmailbox = Args[++i];
				}
				break;
			case 'd':
				if(!(bHasArgAppended || bHasMoreArgs)) throw new Exception("Bad Argument");
				if(bHasArgAppended) {
					domain = '@'+Args[i].substring(2);
				} else {
					domain = '@'+Args[++i];
				}
				break;
			case 't':
				if(!(bHasArgAppended || bHasMoreArgs)) throw new Exception("Bad Argument");
				if(bHasArgAppended) {
					recipient = Args[i].substring(2);
				} else {
					recipient = Args[++i];
				}
				break;
			case 's':
				if(!(bHasArgAppended || bHasMoreArgs)) throw new Exception("Bad Argument");
				if(bHasArgAppended) {
					subject = Args[i].substring(2);
				} else {
					subject = Args[++i];
				}
				break;
			case 'u':
				if(!(bHasArgAppended || bHasMoreArgs)) throw new Exception("Bad Argument");
				if(bHasArgAppended) {
					user = Args[i].substring(2);
				} else {
					user = Args[++i];
				}
				break;
			case 'p':
				if(!(bHasArgAppended || bHasMoreArgs)) throw new Exception("Bad Argument");
				if(bHasArgAppended) {
					password = Args[i].substring(2);
				} else {
					password = Args[++i];
				}
				break;
			case 'v':
				if(bHasArgAppended) throw new Exception("Bad Argument");
				bVerbose = true;
				break;
			default:
				throw new Exception("Bad Argument");
			}
		}
	}

	protected void printUsage() {
		String usage =
			"The following options are supported:\n" +
			"\t-g\tFetch mail\n" +
			"\t-bN\tMailbox to fetch, 1:Inbox, 2:Sent, 3:Saved. Default: 1\n" +
			"\t-PN\tPage to fetch (Default: 0, newest)\n" +
			"\t-n\tFetch new mail only\n" +
			"\t-f<file>\tMailbox to write to.  Default: stdout.\n" +
			"\t-d<domain>\tAppend the domain to user names.\n" +
			"\t-t<recipient>\tRecipient of message being sent.\n" +
			"\t-s<subject>\tSubject line of a message.\n" +
			"\t-u<username>\tLogin username.\n" +
			"\t-p<password>\tPassword.\n" +
//			"\t-v\tVerbose debugging and status messages.\n" +
			"Exactly one of -g or -t must be specified.";
		System.err.println(usage);
	}

	public FetchOK() {
	}

	protected void run(String[] Args) {
		try {
			parseArgs(Args);
		} catch (Exception e) {
			printUsage();
			return;
		}
		if(!(bFetch ^ !recipient.equals(""))) {
			printUsage();
			return;
		}
		okc = new OKConnection(user, password);
		if(bFetch) {	// Were we asked to fetch something?
			fetchMail();
		} else {	// Were we asked to send something?
			sendMail();
		}
	}

	/**
	 * Reinjects messages to OKC.
	 *
	 * Sends messages to the local-part, scraps the domain.
	 * This is BEST EFFORT, without a retry.
	 */
	protected void inject(String s) {
		Mail m = new Mail(okc);
		Message msg = new Message(recipient, "", subject);
		msg.setBody(s);
		if(bVerbose) {
			System.err.println(user + " sending " + subject + " to " + recipient);
		}
		try {
			okc.connect();
			m.sendMessage(msg);
		} catch (IOException ioe) {
			System.err.println(ioe);
		}
	}

	/**
	 * Fetches new, UNREAD messages, and append it to the specified
	 * 	mailbox file.
	 *
	 */
	protected void fetchMail() {
		Writer out; 
		Random r = new Random();
		Vector messages = new Vector();
		Mail m = new Mail(okc);
		String date; 
		String body;
		okc.connect();
		date = new SimpleDateFormat("EEE MMM dd HH:mm:ss yyyy Z").format(new Date());
		try {
			if(localmailbox.equals("") || localmailbox.equals("-")) {
				out = (Writer)new PrintWriter(System.out);
				if(bVerbose) System.err.println("Using mbox to <stdout>");
			} else {
				out = new FileWriter(localmailbox, true);
				if(bVerbose) System.err.println("Using mbox " + localmailbox);
			}

			m.getMailbox(messages, remotemailbox, requestedpage, bNewOnly);
			if(bVerbose) {
				System.err.println("Found " + messages.size());
			}
			for(Enumeration e = messages.elements();
					e.hasMoreElements(); ) {
				Message msg = (Message)e.nextElement();
				m.retrieveBody(msg);
				String txt = Convert.HTML2Text(msg.getBody());
				if(bVerbose) {
					System.err.println("From: " + msg.getFrom());
				}
				body = txt.replaceAll("(\n|^)From ", ">From " );
				out.write("From " + msg.getFrom() + domain + " " + date + "\n");
				out.write("From: " + msg.getFrom() + domain + "\n");
				out.write("To: " + user  + domain + "\n");
				out.write("Date: " + date + "\n");
				out.write("Subject: " + msg.getSubject() + "\n");
				out.write("Content-Length: " + (body.length()+1) + "\n");
				out.write("\n");
				out.write(body);
				out.write("\n\n");
				out.flush();

			}
		} catch (IOException ioe) {
			System.err.println(ioe);
		}
	}

	/**
	 * Reads message from std, proceeds to send it to OKC.
	 */
	protected void sendMail() {
		StringBuffer body = new StringBuffer();
		BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
		String line;
		try {
			while((line=in.readLine()) != null) {
				body.append(line + "\n");
			}
		} catch (IOException ioe) {
			System.err.println("Message Cancelled");
			return;
		}
		inject(body.toString());
	}
}
